%% This script demonstrates over-fitting due to more complex models (M=9).
 % M=9 order polynomial is so flexible that it captures the variations due
 % to noise as well. As a result, it over-fits the data and passes exactly
 % through the data points while failing to capture the real, underlying
 % variation of the data.
 % Less powerful models (M=0, M=1) fail to capture the underlying variations in
 % data.
 % M=3 leads to a polynomial that fails to capture noise variations while
 % capturing the underlying variation in data.
clf;
load data_10;
newx=0:.01:1;

M=0;
[w0,beta0,E_RMS_train0,E_RMS_test0]=fit_polynomial_ML(x,t,M,0);
subplot(221);
plot_fitting_results(newx,w0,x,t);
title(['E_{RMS}=' num2str(E_RMS_train0)]);
text(.2,-1,['$M=$' num2str(M)],'Interpreter','Latex');

M=1;
[w1,beta1,E_RMS_train1,E_RMS_test1]=fit_polynomial_ML(x,t,M,0);
subplot(222);
plot_fitting_results(newx,w1,x,t);
title(['E_{RMS}=' num2str(E_RMS_train1)]);
text(.2,-1,['$M=$' num2str(M)],'Interpreter','Latex');

M=3;
[w3,beta3,E_RMS_train3,E_RMS_test3]=fit_polynomial_ML(x,t,M,0);
subplot(223);
plot_fitting_results(newx,w3,x,t);
title(['E_{RMS}=' num2str(E_RMS_train3)]);
text(.2,-1,['$M=$' num2str(M)],'Interpreter','Latex');

M=9;
[w9,beta9,E_RMS_train9,E_RMS_test9]=fit_polynomial_ML(x,t,M,0);
subplot(224);
plot_fitting_results(newx,w9,x,t);
title(['E_{RMS}=' num2str(E_RMS_train9)]);
text(.2,-1,['$M=$' num2str(M)],'Interpreter','Latex');

print(gcf,'Figure_1_4','-dpng');

%% fitted polynomial coefficients (similar to Bishop, Table 1.1)
 % w9 corresponding to M=9 has large magnitudes which is a sign of over-fitting
 % w3 corresponding to M=3 has smaller magnitudes and is a better fit of
 % the real function because it does not have the power to capture the
 % variations due to noise.
 % w0 and w1 have even smaller magnitudes and do not fit the data well
coefficients=[[w0;zeros(9,1)]  [w1;zeros(8,1)] [w3;zeros(6,1)] w9];
disp(vpa(coefficients,5));
save coefficients.mat coefficients